@@ -1,6 +1,6 @@
 #include "config.h"
 
-#define FUSE_USE_VERSION 26
+#define FUSE_USE_VERSION 35
 
 #include <glib-unix.h>
 
@@ -88,7 +88,7 @@
 
 static GThread *fuse_thread = NULL;
 static struct fuse_session *session = NULL;
-static struct fuse_chan *main_ch = NULL;
+G_LOCK_DEFINE (session);
 static char *mount_path = NULL;
 static pthread_t fuse_pthread = 0;
 static uid_t my_uid;
@@ -230,7 +230,7 @@ app_can_write_doc (PermissionDbEntry *entry, const char *app_id)
   if (app_id == NULL)
     return TRUE;
 
-  if (document_entry_has_permissions (entry, app_id, DOCUMENT_PERMISSION_FLAGS_WRITE))
+  if (document_entry_has_permissions_by_app_id (entry, app_id, DOCUMENT_PERMISSION_FLAGS_WRITE))
     return TRUE;
 
   return FALSE;
@@ -242,7 +242,7 @@ app_can_see_doc (PermissionDbEntry *entry, const char *app_id)
   if (app_id == NULL)
     return TRUE;
 
-  if (document_entry_has_permissions (entry, app_id, DOCUMENT_PERMISSION_FLAGS_READ))
+  if (document_entry_has_permissions_by_app_id (entry, app_id, DOCUMENT_PERMISSION_FLAGS_READ))
     return TRUE;
 
   return FALSE;
@@ -349,6 +349,31 @@ setattr_flags_to_string (int flags)
   return g_string_free (s, FALSE);
 }
 
+static char *
+renameat2_flags_to_string (int flags)
+{
+#if HAVE_RENAMEAT2
+  GString *s = g_string_new ("");
+
+  if (flags & RENAME_EXCHANGE)
+    g_string_append (s, "EXCHANGE,");
+
+  if (flags & RENAME_NOREPLACE)
+    g_string_append (s, "NOREPLACE,");
+
+  if (flags & RENAME_WHITEOUT)
+    g_string_append (s, "WHITEOUT,");
+
+  /* Remove last comma */
+  if (s->len > 0)
+    g_string_truncate (s, s->len - 1);
+
+  return g_string_free (s, FALSE);
+#else
+  return g_strdup_printf ("%#x", flags);
+#endif
+}
+
 static guint
 devino_hash  (gconstpointer  key)
 {
@@ -1668,8 +1693,10 @@ invalidate_doc_domain (gpointer user_data)
 
   parent_ino = xdp_inode_to_ino (doc_domain->parent_inode);
 
-  if (g_atomic_int_get (&doc_domain->parent_inode->kernel_ref_count) > 0 && main_ch != NULL)
-    fuse_lowlevel_notify_inval_entry (main_ch, parent_ino, doc_domain->doc_id, strlen (doc_domain->doc_id));
+  XDP_AUTOLOCK (session);
+  if (session && g_atomic_int_get (&doc_domain->parent_inode->kernel_ref_count) > 0)
+    fuse_lowlevel_notify_inval_entry (session, parent_ino, doc_domain->doc_id,
+                                      strlen (doc_domain->doc_id));
 
   return FALSE;
 }
@@ -2460,15 +2487,39 @@ xdp_fuse_unlink (fuse_req_t  req,
   xdp_reply_err (op, req, 0);
 }
 
+static int
+try_renameat (int olddirfd,
+              const char *oldpath,
+              int newdirfd,
+              const char *newpath,
+              unsigned int flags)
+{
+#if HAVE_RENAMEAT2
+  return renameat2 (olddirfd, oldpath, newdirfd, newpath, flags);
+#else
+  if (flags)
+    {
+      g_warning ("renameat2 is not supported by this system and rename flags are set");
+      errno = EINVAL;
+      return -1;
+    }
+
+  return renameat (olddirfd, oldpath, newdirfd, newpath);
+#endif
+}
+
+
 static void
 xdp_fuse_rename (fuse_req_t  req,
                  fuse_ino_t  parent_ino,
                  const char *name,
                  fuse_ino_t  newparent_ino,
-                 const char *newname)
+                 const char *newname,
+                 unsigned int flags)
 {
   g_autoptr(XdpInode) parent = xdp_inode_from_ino (parent_ino);
   g_autoptr(XdpInode) newparent = xdp_inode_from_ino (newparent_ino);
+  g_autofree char *rename_flags_string = renameat2_flags_to_string (flags);
   XdpDomain *domain;
   int res, errsv;
   int olddirfd, newdirfd, dirfd;
@@ -2476,7 +2527,8 @@ xdp_fuse_rename (fuse_req_t  req,
   xdp_autofd int close_fd2 = -1;
   const char *op = "RENAME";
 
-  g_debug ("RENAME %lx %s -> %lx %s", parent_ino, name, newparent_ino, newname);
+  g_debug ("RENAME %lx %s -> %lx %s (flags: %s)", parent_ino, name,
+           newparent_ino, newname, rename_flags_string);
 
   if (!xdp_document_inode_checks (op, req, parent,
                                   CHECK_CAN_WRITE |
@@ -2498,7 +2550,7 @@ xdp_fuse_rename (fuse_req_t  req,
       if (newdirfd < 0)
         return xdp_reply_err (op, req, -newdirfd);
 
-      res = renameat (olddirfd, name, newdirfd, newname);
+      res = try_renameat (olddirfd, name, newdirfd, newname, flags);
       if (res != 0)
         return xdp_reply_err (op, req, errno);
 
@@ -2532,7 +2584,7 @@ xdp_fuse_rename (fuse_req_t  req,
           close (tmp_fd);
 
           g_mutex_lock (&domain->tempfile_mutex);
-          res = renameat (dirfd, name, dirfd, tmpname);
+          res = try_renameat (dirfd, name, dirfd, tmpname, flags);
           if (res == -1)
             {
               res = -errno;
@@ -2563,7 +2615,7 @@ xdp_fuse_rename (fuse_req_t  req,
             {
               XdpTempfile *tempfile = stolen_value;
 
-              res = renameat (dirfd, tempfile->tempname, dirfd, newname);
+              res = try_renameat (dirfd, tempfile->tempname, dirfd, newname, flags);
               errsv = errno;
 
               if (res == -1) /* Revert tempfile steal */
@@ -3004,20 +3056,26 @@ xdp_fuse_init_cb (void                  *userdata,
                   struct fuse_conn_info *conn)
 {
   g_debug ("INIT");
+
+  /* splice_read: use splice() to read from fuse pipe */
+  conn->want |= FUSE_CAP_SPLICE_READ;
+  /* splice_write: use splice() to write to fuse pipe */
+  conn->want |= FUSE_CAP_SPLICE_WRITE;
+  /* splice_move: move buffers from writing app to kernel during splice write */
+  conn->want |= FUSE_CAP_SPLICE_MOVE;
+  /* atomic_o_trunc: We handle O_TRUNC in create() */
+  conn->want |= FUSE_CAP_ATOMIC_O_TRUNC;
 }
 
-extern void on_fuse_unmount (void);
-
-static int destroyed;
+extern int on_fuse_unmount (void);
 
 static void
 xdp_fuse_destroy_cb (void *userdata)
 {
   g_debug ("DESTROY");
 
-  destroyed = 1;
-
-  on_fuse_unmount ();
+  /* Ensure we call this on the main thread */
+  g_idle_add ((GSourceFunc) on_fuse_unmount, NULL);
 }
 
 static struct fuse_lowlevel_ops xdp_fuse_oper = {
@@ -3059,14 +3117,19 @@ static struct fuse_lowlevel_ops xdp_fuse_oper = {
  .fallocate    = xdp_fuse_fallocate,
 };
 
-static gpointer
-xdp_fuse_mainloop (gpointer data)
+typedef struct {
+  GMutex lock;
+  GCond cond;
+  GError *error;
+} XdpFuseThreadData;
+
+static void
+xdp_fuse_mainloop (struct fuse_session *se,
+                   struct fuse_loop_config *loop_config)
 {
   const char *status;
 
-  fuse_pthread = pthread_self ();
-
-  fuse_session_loop_mt (session);
+  fuse_session_loop_mt (se, loop_config);
 
   status = getenv ("TEST_DOCUMENT_PORTAL_FUSE_STATUS");
   if (status)
@@ -3079,11 +3142,79 @@ xdp_fuse_mainloop (gpointer data)
       g_file_set_contents (status, s->str, -1, &error);
       g_assert_no_error (error);
     }
+}
 
-  fuse_session_remove_chan (main_ch);
-  fuse_session_destroy (session);
-  fuse_unmount (mount_path, main_ch);
-  main_ch = NULL;
+typedef struct fuse_args XdpAutoFuseArgs;
+G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC (XdpAutoFuseArgs, fuse_opt_free_args);
+
+static gpointer
+xdp_fuse_thread (gpointer data)
+{
+  /* Options:
+   *  auto_unmount: Tell fusermount to auto unmount if we die.
+   */
+  static char *fusermount_argv[] = {
+    "xdp-fuse", "-osubtype=portal,fsname=portal,auto_unmount",
+  };
+  g_auto(XdpAutoFuseArgs) args =
+    FUSE_ARGS_INIT (G_N_ELEMENTS (fusermount_argv), fusermount_argv);
+  g_autoptr(GMutexLocker) locker = NULL;
+  g_autoptr(GMutexLocker) session_locker = NULL;
+  const char *path;
+  struct fuse_session *se;
+  XdpFuseThreadData *thread_data = data;
+  struct fuse_cmdline_opts opts = {0};
+  struct fuse_loop_config loop_config = {0};
+
+  locker = g_mutex_locker_new (&thread_data->lock);
+  fuse_pthread = pthread_self ();
+
+  g_cond_signal (&thread_data->cond);
+
+  if (fuse_parse_cmdline (&args, &opts) != 0)
+    {
+      g_set_error (&thread_data->error, XDG_DESKTOP_PORTAL_ERROR,
+                   XDG_DESKTOP_PORTAL_ERROR_INVALID_ARGUMENT,
+                   "Impossible to parse command line");
+      return NULL;
+    }
+
+  se = fuse_session_new (&args, &xdp_fuse_oper,
+                         sizeof (xdp_fuse_oper), NULL);
+  if (se == NULL)
+    {
+      g_set_error (&thread_data->error, XDG_DESKTOP_PORTAL_ERROR,
+                   XDG_DESKTOP_PORTAL_ERROR_FAILED,
+                   "Can't create fuse session");
+      return NULL;
+    }
+
+  path = xdp_fuse_get_mountpoint ();
+  if (fuse_session_mount (se, path) != 0)
+    {
+      fuse_session_destroy (se);
+      g_set_error (&thread_data->error, XDG_DESKTOP_PORTAL_ERROR,
+                   XDG_DESKTOP_PORTAL_ERROR_FAILED,
+                   "Can't mount path %s", path);
+      return NULL;
+    }
+
+  session = se;
+  thread_data = NULL;
+  g_clear_pointer (&locker, g_mutex_locker_free);
+
+  loop_config.clone_fd = opts.clone_fd;
+  loop_config.max_idle_threads = opts.max_idle_threads;
+  thread_data = NULL;
+
+  session_locker = g_mutex_locker_new (&G_LOCK_NAME (session));
+  g_clear_pointer (&session_locker, g_mutex_locker_free);
+  xdp_fuse_mainloop (session, &loop_config);
+
+  session_locker = g_mutex_locker_new (&G_LOCK_NAME (session));
+  fuse_session_unmount (se);
+  fuse_session_destroy (se);
+  session = NULL;
 
   return NULL;
 }
@@ -3091,16 +3222,7 @@ xdp_fuse_mainloop (gpointer data)
 gboolean
 xdp_fuse_init (GError **error)
 {
-  /* Options:
-   *  auto_unmount: Tell fusermount to auto unmount if we die.
-   *  splice_read: use splice() to read from fuse pipe
-   *  splice_write: use splice() to write to fuse pipe
-   *  splice_move: move buffers from writing app to kernel during splice write
-   *  atomic_o_trunc: We handle O_TRUNC in create()
-   *  big_writes: Allow > 4k writes
-   */
-  char *fusermount_argv[] = { "xdp-fuse", "-osubtype=portal,fsname=portal,auto_unmount,splice_read,splice_write,splice_move,atomic_o_trunc,big_writes" };
-  struct fuse_args args = FUSE_ARGS_INIT (G_N_ELEMENTS (fusermount_argv), fusermount_argv);
+  XdpFuseThreadData thread_data = {0};
   struct stat st;
   struct statfs stfs;
   const char *path;
@@ -3137,7 +3259,7 @@ xdp_fuse_init (GError **error)
        (statfs_res == 0 && stfs.f_type == 0x65735546 /* fuse */)))
     {
       int count;
-      char *umount_argv[] = { "fusermount", "-u", "-z", (char *) path, NULL };
+      char *umount_argv[] = { "fusermount3", "-u", "-z", (char *) path, NULL };
 
       g_spawn_sync (NULL, umount_argv, NULL, G_SPAWN_SEARCH_PATH,
                     NULL, NULL, NULL, NULL, NULL, NULL);
@@ -3155,29 +3277,27 @@ xdp_fuse_init (GError **error)
       return FALSE;
     }
 
-  main_ch = fuse_mount (path, &args);
-  if (main_ch == NULL)
+  g_mutex_init (&thread_data.lock);
+  g_cond_init (&thread_data.cond);
+
+  g_mutex_lock (&thread_data.lock);
+  XDP_AUTOLOCK (session);
+  fuse_thread = g_thread_new ("fuse mainloop", xdp_fuse_thread, &thread_data);
+
+  while (session == NULL && thread_data.error == NULL)
+    g_cond_wait (&thread_data.cond, &thread_data.lock);
+
+  g_mutex_unlock (&thread_data.lock);
+  g_cond_clear (&thread_data.cond);
+  g_mutex_clear (&thread_data.lock);
+
+  if (thread_data.error != NULL)
     {
-      fuse_opt_free_args (&args);
-      g_set_error (error, XDG_DESKTOP_PORTAL_ERROR, XDG_DESKTOP_PORTAL_ERROR_FAILED,
-                   "Can't mount fuse fs on %s: %s", path, g_strerror (errno));
+      g_propagate_error (error, g_steal_pointer (&thread_data.error));
       return FALSE;
     }
 
-  session = fuse_lowlevel_new (&args, &xdp_fuse_oper,
-                               sizeof (xdp_fuse_oper), NULL);
-  if (session == NULL)
-    {
-      fuse_opt_free_args (&args);
-      g_set_error (error, XDG_DESKTOP_PORTAL_ERROR, XDG_DESKTOP_PORTAL_ERROR_FAILED,
-                   "Can't create fuse session");
-      return FALSE;
-    }
-  fuse_session_add_chan (session, main_ch);
-
-  fuse_thread = g_thread_new ("fuse mainloop", xdp_fuse_mainloop, session);
-
-  fuse_opt_free_args (&args);
+  g_assert (session != NULL);
 
   return TRUE;
 }
@@ -3185,14 +3305,18 @@ xdp_fuse_init (GError **error)
 void
 xdp_fuse_exit (void)
 {
-  if (!destroyed && session)
-    fuse_session_exit (session);
+  {
+    XDP_AUTOLOCK (session);
 
-  if (fuse_pthread)
-    pthread_kill (fuse_pthread, SIGHUP);
+    if (session)
+      fuse_session_exit (session);
 
-  if (fuse_thread)
-    g_thread_join (fuse_thread);
+    if (fuse_pthread)
+      pthread_kill (fuse_pthread, SIGHUP);
+  }
+
+  g_clear_pointer (&fuse_thread, g_thread_join);
+  g_assert (session == NULL);
 }
 
 const char *
@@ -3239,11 +3363,12 @@ xdp_fuse_invalidate_doc_app (const char *doc_id,
                              const char *opt_app_id)
 {
   g_autoptr(GArray) invalidates = NULL;
+  XDP_AUTOLOCK (session);
   int i;
 
   /* This can happen if fuse is not initialized yet for the very
      first dbus message that activated the service */
-  if (main_ch == NULL)
+  if (session == NULL)
     return;
 
   g_debug ("invalidate %s/%s", doc_id, opt_app_id ? opt_app_id : "*");
@@ -3276,12 +3401,12 @@ xdp_fuse_invalidate_doc_app (const char *doc_id,
 
       if (invalidate->filename)
         {
-          fuse_lowlevel_notify_inval_entry (main_ch, invalidate->ino,
+          fuse_lowlevel_notify_inval_entry (session, invalidate->ino,
                                             invalidate->filename, strlen (invalidate->filename));
           g_free (invalidate->filename);
         }
       else
-        fuse_lowlevel_notify_inval_inode (main_ch, invalidate->ino, 0, 0);
+        fuse_lowlevel_notify_inval_inode (session, invalidate->ino, 0, 0);
     }
 }
 
