@@ -15,7 +15,9 @@ void        xdp_fuse_exit (void);
 const char *xdp_fuse_get_mountpoint (void);
 void        xdp_fuse_invalidate_doc_app (const char *doc_id,
                                          const char *opt_app_id);
-char      *xdp_fuse_lookup_id_for_inode (ino_t inode);
+char      *xdp_fuse_lookup_id_for_inode (ino_t    inode,
+                                         gboolean directory,
+                                         char   **real_path_out);
 
 
 G_END_DECLS
