@@ -30,7 +30,7 @@ Permission device_get_permission_sync (const char *app_id,
 
 gboolean device_query_permission_sync (const char *app_id,
                                        const char *device,
-                                       const char *request_handle);
+                                       Request    *request);
 
 GDBusInterfaceSkeleton * device_create (GDBusConnection *connection,
                                         const char      *dbus_name,
