@@ -243,7 +243,7 @@ save_icon_and_get_desktop_entry (const char  *desktop_file_id,
     else
       subdir = g_strdup_printf ("%sx%s", icon_size, icon_size);
 
-    icon_subdir = g_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "icons", subdir, NULL);
+    icon_subdir = g_build_filename (g_get_user_data_dir (), XDG_PORTAL_ICONS_DIR, subdir, NULL);
     icon_path = g_build_filename (icon_subdir, icon_name, NULL);
 
     g_key_file_set_string (key_file, G_KEY_FILE_DESKTOP_GROUP, "Icon", icon_path);
@@ -375,7 +375,7 @@ handle_install (XdpDynamicLauncher    *object,
   /* Put the desktop file in ~/.local/share/xdg-desktop-portal/applications/ so
    * there's no ambiguity about which launchers were created by this portal.
    */
-  desktop_dir = g_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "applications", NULL);
+  desktop_dir = g_build_filename (g_get_user_data_dir (), XDG_PORTAL_APPLICATIONS_DIR, NULL);
   g_mkdir_with_parents (desktop_dir, 0700);
   desktop_path = g_build_filename (desktop_dir, arg_desktop_file_id, NULL);
   if (!g_key_file_save_to_file (desktop_keyfile, desktop_path, &error))
@@ -386,7 +386,7 @@ handle_install (XdpDynamicLauncher    *object,
    */
   link_path = g_build_filename (g_get_user_data_dir (), "applications", arg_desktop_file_id, NULL);
   link_file = g_file_new_for_path (link_path);
-  relative_path = g_build_filename ("..", "xdg-desktop-portal", "applications", arg_desktop_file_id, NULL);
+  relative_path = g_build_filename ("..", XDG_PORTAL_APPLICATIONS_DIR, arg_desktop_file_id, NULL);
   g_file_delete (link_file, NULL, NULL);
   if (!g_file_make_symbolic_link (link_file, relative_path, NULL, &error))
     goto error;
@@ -751,8 +751,8 @@ handle_uninstall (XdpDynamicLauncher    *object,
       return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
-  icon_dir = g_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "icons", NULL);
-  desktop_dir = g_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "applications", NULL);
+  icon_dir = g_build_filename (g_get_user_data_dir (), XDG_PORTAL_ICONS_DIR, NULL);
+  desktop_dir = g_build_filename (g_get_user_data_dir (), XDG_PORTAL_APPLICATIONS_DIR, NULL);
 
   link_file = g_file_new_build_filename (g_get_user_data_dir (), "applications", arg_desktop_file_id, NULL);
   if (!g_file_delete (link_file, NULL, &error))
@@ -803,7 +803,7 @@ handle_get_desktop_entry (XdpDynamicLauncher    *object,
   if (!validate_desktop_file_id (app_id, arg_desktop_file_id, &error))
     goto error;
 
-  desktop_dir = g_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "applications", NULL);
+  desktop_dir = g_build_filename (g_get_user_data_dir (), XDG_PORTAL_APPLICATIONS_DIR, NULL);
 
   desktop_path = g_build_filename (desktop_dir, arg_desktop_file_id, NULL);
   if (!g_file_get_contents (desktop_path, &contents, &length, &error))
@@ -852,8 +852,8 @@ handle_get_icon (XdpDynamicLauncher    *object,
   if (!validate_desktop_file_id (app_id, arg_desktop_file_id, &error))
     goto error;
 
-  desktop_dir = g_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "applications", NULL);
-  icon_dir = g_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "icons", NULL);
+  desktop_dir = g_build_filename (g_get_user_data_dir (), XDG_PORTAL_APPLICATIONS_DIR, NULL);
+  icon_dir = g_build_filename (g_get_user_data_dir (), XDG_PORTAL_ICONS_DIR, NULL);
 
   desktop_path = g_build_filename (desktop_dir, arg_desktop_file_id, NULL);
   if (!g_file_get_contents (desktop_path, &contents, &length, &error))
@@ -957,7 +957,7 @@ handle_launch (XdpDynamicLauncher    *object,
   if (!validate_desktop_file_id (app_id, arg_desktop_file_id, &error))
     goto error;
 
-  desktop_dir = g_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "applications", NULL);
+  desktop_dir = g_build_filename (g_get_user_data_dir (), XDG_PORTAL_APPLICATIONS_DIR, NULL);
 
   desktop_path = g_build_filename (desktop_dir, arg_desktop_file_id, NULL);
   if (!g_file_test (desktop_path, G_FILE_TEST_EXISTS))
@@ -999,219 +999,6 @@ error:
   return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
-static char *
-find_renamed_app_id (const char *old_app_id)
-{
-  g_autofree char *renamed_to = NULL;
-  GList *app_infos = g_app_info_get_all ();
-  for (GList *l = app_infos; l; l = l->next)
-    {
-      GDesktopAppInfo *info = l->data;
-      g_auto(GStrv) renamed_from = NULL;
-      renamed_from = g_desktop_app_info_get_string_list (info, "X-Flatpak-RenamedFrom", NULL);
-      if (renamed_from == NULL)
-        continue;
-      if (!g_strv_contains ((const char * const *)renamed_from, old_app_id))
-        continue;
-
-      renamed_to = g_desktop_app_info_get_string (info, "X-Flatpak");
-      break;
-    }
-
-  g_list_free_full (app_infos, g_object_unref);
-  return g_steal_pointer (&renamed_to);
-}
-
-/*
- * It's possible an app was renamed using Flatpak's end-of-life-rebase
- * mechanism, and either (a) the app was installed system-wide and the update
- * was applied by another user, so the migration for this user has to happen
- * when x-d-p launches at the start of the session, or (b) the version of
- * Flatpak is not new enough for the migration of the launchers to be handled
- * by Flatpak, so x-d-p has to do it.
- *
- * This function also handles deleting the launchers in case the parent app has
- * been uninstalled.
- */
-static void
-migrate_renamed_app_launchers (void)
-{
-  g_autoptr(GFile) desktop_dir = NULL;
-  g_autoptr(GFileEnumerator) children = NULL;
-  g_autoptr(GError) error = NULL;
-  g_autofree char *icon_dir = NULL;
-
-  desktop_dir = g_file_new_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "applications", NULL);
-  children = g_file_enumerate_children (desktop_dir, "standard::name", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &error);
-  if (children == NULL)
-    {
-      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
-        g_warning ("Error encountered enumerating launchers: %s", error->message);
-      return;
-    }
-
-  icon_dir = g_build_filename (g_get_user_data_dir (), "xdg-desktop-portal", "icons", NULL);
-  for (;;)
-    {
-      g_autoptr(GFileInfo) info = g_file_enumerator_next_file (children, NULL, NULL);
-      const char *desktop_name;
-      g_autofree char *desktop_path = NULL;
-      g_autofree char *icon_path = NULL;
-      g_autofree char *tryexec_path = NULL;
-      g_autofree char *app_id = NULL;
-      g_autofree char *renamed_to = NULL;
-      g_autoptr(GKeyFile) key_file = NULL;
-      g_autoptr(GFile) icon_file = NULL;
-      g_autoptr(GFile) desktop_file = NULL;
-
-      if (info == NULL)
-        return;
-
-      desktop_name = g_file_info_get_name (info);
-      if (!g_str_has_suffix (desktop_name, ".desktop"))
-        continue;
-
-      desktop_path = g_build_filename (g_file_peek_path (desktop_dir), desktop_name, NULL);
-      key_file = g_key_file_new ();
-      if (!g_key_file_load_from_file (key_file, desktop_path,
-                                      G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
-                                      &error))
-        {
-          g_warning ("Error encountered loading key file %s: %s", desktop_path, error->message);
-          continue;
-        }
-
-      tryexec_path = g_key_file_get_string (key_file, G_KEY_FILE_DESKTOP_GROUP, "TryExec", NULL);
-      if (tryexec_path == NULL)
-        continue;
-      if (!g_path_is_absolute (tryexec_path))
-        {
-          /* Here we're just checking for existence not a renamed binary */
-          renamed_to = g_find_program_in_path (tryexec_path);
-          if (renamed_to)
-            continue;
-        }
-      else if (g_file_test (tryexec_path, G_FILE_TEST_IS_EXECUTABLE))
-        {
-          continue;
-        }
-      else if (g_key_file_has_key (key_file, G_KEY_FILE_DESKTOP_GROUP, "X-Flatpak", NULL))
-        {
-          app_id = g_path_get_basename (tryexec_path);
-          if (strlen(app_id) < 2 ||
-              !g_dbus_is_name (app_id) ||
-              !g_str_has_prefix (desktop_name, app_id))
-            {
-              g_warning ("Unable to determine app id for %s", desktop_name);
-              continue;
-            }
-          renamed_to = find_renamed_app_id (app_id);
-        }
-
-      icon_path = g_key_file_get_string (key_file, G_KEY_FILE_DESKTOP_GROUP, "Icon", NULL);
-      if (g_str_has_prefix (icon_path, icon_dir))
-        icon_file = g_file_new_for_path (icon_path);
-
-      desktop_file = g_file_new_for_path (desktop_path);
-
-      if (renamed_to == NULL)
-        {
-          g_autoptr(GFile) link_file = NULL;
-
-          link_file = g_file_new_build_filename (g_get_user_data_dir (), "applications", desktop_name, NULL);
-          if (!g_file_delete (link_file, NULL, &error))
-            {
-              g_warning ("Couldn't delete sym link %s: %s", g_file_peek_path (link_file), error->message);
-              g_clear_error (&error);
-            }
-
-          if (!g_file_delete (desktop_file, NULL, &error))
-            {
-              g_warning ("Couldn't delete desktop file %s: %s", g_file_peek_path (desktop_file), error->message);
-              g_clear_error (&error);
-            }
-
-          if (icon_file && !g_file_delete (icon_file, NULL, &error))
-            {
-              g_warning ("Couldn't delete icon file %s: %s", g_file_peek_path (icon_file), error->message);
-              g_clear_error (&error);
-            }
-        }
-      else /* renamed_to != NULL */
-        {
-          g_autoptr(GFile) link_file = NULL;
-          g_autoptr(GString) data_string = NULL;
-          g_autoptr(GKeyFile) new_key_file = NULL;
-          g_autofree char *new_desktop = NULL;
-          g_autofree char *new_icon = NULL;
-          g_autofree char *icon_basename = NULL;
-          g_autofree char *link_path = NULL;
-          g_autofree char *relative_path = NULL;
-          g_autofree char *old_data = NULL;
-          const gchar *desktop_suffix;
-          gchar *icon_suffix;
-
-          if (!g_key_file_has_key (key_file, G_KEY_FILE_DESKTOP_GROUP, "X-Flatpak", NULL))
-            g_assert_not_reached ();
-
-          /* Fix paths in desktop file with a find-and-replace. */
-          old_data = g_key_file_to_data (key_file, NULL, NULL);
-          data_string = g_string_new ((const char *)old_data);
-          g_string_replace (data_string, app_id, renamed_to, 0);
-          new_key_file = g_key_file_new ();
-          if (!g_key_file_load_from_data (new_key_file, data_string->str, -1,
-                                          G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
-                                          &error))
-            {
-              g_warning ("Cannot load desktop file %s after rewrite: %s", desktop_path, error->message);
-              g_warning ("Key file contents:\n%s\n", (const char *)data_string->str);
-              continue;
-            }
-          if (!g_key_file_save_to_file (new_key_file, desktop_path, &error))
-            {
-              g_warning ("Couldn't rewrite desktop file %s: %s", desktop_path, error->message);
-              continue;
-            }
-
-          /* Rename desktop file */
-          desktop_suffix = desktop_name + strlen (app_id);
-          new_desktop = g_strconcat (renamed_to, desktop_suffix, NULL);
-          if (!g_file_set_display_name (desktop_file, new_desktop, NULL, &error))
-            {
-              g_warning ("Unable to rename desktop file %s -> %s: %s", desktop_name, new_desktop, error->message);
-              continue;
-            }
-
-          /* Fix symlink */
-          link_path = g_build_filename (g_get_user_data_dir (), "applications", desktop_name, NULL);
-          link_file = g_file_new_for_path (link_path);
-          relative_path = g_build_filename ("..", "xdg-desktop-portal", "applications", new_desktop, NULL);
-          g_file_delete (link_file, NULL, NULL);
-          if (!g_file_make_symbolic_link (link_file, relative_path, NULL, &error))
-            {
-              g_warning ("Unable to rename desktop file %s -> %s: %s", desktop_name, new_desktop, error->message);
-              continue;
-            }
-
-          /* And rename the icon */
-          if (icon_file)
-            {
-              icon_basename = g_path_get_basename (icon_path);
-              if (!g_str_has_prefix (icon_basename, app_id))
-                continue;
-
-              icon_suffix = icon_basename + strlen (app_id);
-              new_icon = g_strconcat (renamed_to, icon_suffix, NULL);
-              if (!g_file_set_display_name (icon_file, new_icon, NULL, &error))
-                {
-                  g_warning ("Unable to rename icon file %s -> %s: %s", icon_basename, new_icon, error->message);
-                  continue;
-                }
-            }
-        }
-    }
-}
-
 static void
 dynamic_launcher_iface_init (XdpDynamicLauncherIface *iface)
 {
@@ -1231,7 +1018,6 @@ dynamic_launcher_init (DynamicLauncher *dl)
   g_object_bind_property (G_OBJECT (impl), "supported-launcher-types",
                           G_OBJECT (dl), "supported-launcher-types",
                           G_BINDING_SYNC_CREATE);
-  migrate_renamed_app_launchers ();
 }
 
 static void
