@@ -31,7 +31,6 @@
 #include <fcntl.h>
 
 #include <gio/gio.h>
-#include <gio/gunixfdlist.h>
 
 #include "file-chooser.h"
 #include "request.h"
@@ -114,7 +113,11 @@ send_response_in_thread_func (GTask        *task,
           g_autofree char *ruri = NULL;
           g_autoptr(GError) error = NULL;
 
-          ruri = register_document (uris[i], xdp_app_info_get_id (request->app_info), for_save, writable, directory, &error);
+          if (xdp_app_info_is_host (request->app_info))
+            ruri = g_strdup (uris[i]);
+          else
+            ruri = register_document (uris[i], xdp_app_info_get_id (request->app_info), for_save, writable, directory, &error);
+
           if (ruri == NULL)
             {
               g_warning ("Failed to register %s: %s", uris[i], error->message);
@@ -154,6 +157,7 @@ open_file_done (GObject *source,
                                                     result,
                                                     &error))
     {
+      g_dbus_error_strip_remote_error (error);
       g_warning ("Backend call failed: %s", error->message);
     }
 
@@ -476,7 +480,7 @@ handle_open_file (XdpFileChooser *object,
                            &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   impl_request = xdp_impl_request_proxy_new_sync (g_dbus_proxy_get_connection (G_DBUS_PROXY (impl)),
@@ -487,7 +491,7 @@ handle_open_file (XdpFileChooser *object,
   if (!impl_request)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   dir_option = g_variant_lookup_value (arg_options,
@@ -511,7 +515,7 @@ handle_open_file (XdpFileChooser *object,
 
   xdp_file_chooser_complete_open_file (object, invocation, request->id);
 
-  return TRUE;
+  return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
 static XdpOptionKey save_file_options[] = {
@@ -542,6 +546,7 @@ save_file_done (GObject *source,
                                                     result,
                                                     &error))
     {
+      g_dbus_error_strip_remote_error (error);
       g_warning ("Backend call failed: %s", error->message);
     }
 
@@ -576,7 +581,7 @@ handle_save_file (XdpFileChooser *object,
                                              XDG_DESKTOP_PORTAL_ERROR,
                                              XDG_DESKTOP_PORTAL_ERROR_NOT_ALLOWED,
                                              "File saving disabled");
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   REQUEST_AUTOLOCK (request);
@@ -587,7 +592,7 @@ handle_save_file (XdpFileChooser *object,
                            &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   impl_request = xdp_impl_request_proxy_new_sync (g_dbus_proxy_get_connection (G_DBUS_PROXY (impl)),
@@ -598,7 +603,7 @@ handle_save_file (XdpFileChooser *object,
   if (!impl_request)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   g_object_set_data (G_OBJECT (request), "for-save", GINT_TO_POINTER (TRUE));
@@ -618,7 +623,7 @@ handle_save_file (XdpFileChooser *object,
 
   xdp_file_chooser_complete_open_file (object, invocation, request->id);
 
-  return TRUE;
+  return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
 static XdpOptionKey save_files_options[] = {
@@ -647,6 +652,7 @@ save_files_done (GObject *source,
                                                      result,
                                                      &error))
     {
+      g_dbus_error_strip_remote_error (error);
       g_warning ("Backend call failed: %s", error->message);
     }
 
@@ -679,7 +685,7 @@ handle_save_files (XdpFileChooser *object,
                                              XDG_DESKTOP_PORTAL_ERROR,
                                              XDG_DESKTOP_PORTAL_ERROR_NOT_ALLOWED,
                                              "File saving disabled");
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   REQUEST_AUTOLOCK (request);
@@ -690,7 +696,7 @@ handle_save_files (XdpFileChooser *object,
                            &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   impl_request = xdp_impl_request_proxy_new_sync (g_dbus_proxy_get_connection (G_DBUS_PROXY (impl)),
@@ -701,7 +707,7 @@ handle_save_files (XdpFileChooser *object,
   if (!impl_request)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   g_object_set_data (G_OBJECT (request), "for-save", GINT_TO_POINTER (TRUE));
@@ -721,7 +727,7 @@ handle_save_files (XdpFileChooser *object,
 
   xdp_file_chooser_complete_open_file (object, invocation, request->id);
 
-  return TRUE;
+  return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
 static void
